package com.poststats.golf.rs.api;

import java.util.List;

import com.poststats.golf.rs.api.model.Course;
import com.poststats.rs.api.model.PagedCollection;
import com.poststats.rs.api.model.Pagination;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.Parameters;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.validation.Valid;
import jakarta.validation.constraints.DecimalMax;
import jakarta.validation.constraints.DecimalMin;
import jakarta.validation.constraints.Max;
import jakarta.validation.constraints.Positive;
import jakarta.ws.rs.BeanParam;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;

/**
 * @author brian.long@poststats.com
 */
@Path("/golf/courses")
@Tag(name = "Course API")
@ApiResponses({
		@ApiResponse(responseCode = "200", description = "Success"),
		@ApiResponse(responseCode = "400", description = "A query parameter is not valid"),
		@ApiResponse(responseCode = "404", description = "No matching golf courses were found")
})
public interface CoursesApi {

	@GET
	@Path("/byName")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Searches for golf courses by their name.",
			description = "Searches for golf courses by their name, excluding prefix (e.g. The) and suffix (e.g. Golf Club), returning limited meta-data about each matched golf course."
	)
	@Parameters({
			@Parameter(
					name = "name",
					required = true,
					description = "A fragment or whole golf course name; the prefix and suffix are not included"
			)
	})
	@Tag(name = "Search API")
	PagedCollection<List<Course>> searchByName(@QueryParam("name") String name, @BeanParam @Valid Pagination paging);

	@GET
	@Path("/byLocation/{country}/{state}")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Searches for golf courses by their location.",
			description = "Searches for golf courses by their major jurisdiction (e.g. US States), returning limited meta-data about each matched golf course."
	)
	@Parameters({
			@Parameter(name = "country", required = true, description = "A country code", example = "US"),
			@Parameter(name = "state", description = "A State or high-level jurisdiction", example = "FL")
	})
	@Tag(name = "Search API")
	PagedCollection<List<Course>> searchByJurisdiction(@PathParam("country") String country,
			@PathParam("state") String state, @BeanParam @Valid Pagination paging);

	@GET
	@Path("/byGeo")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Searches for golf courses by their location.",
			description = "Searches for golf courses by their major jurisdiction (e.g. US States), returning limited meta-data about each matched golf course."
	)
	@Parameters({
			@Parameter(
					name = "latitude",
					required = true,
					description = "A latitude in decimal degrees",
					example = "41.2390"
			),
			@Parameter(
					name = "longitude",
					required = true,
					description = "A longitude in decimal degrees",
					example = "-81.34891"
			), @Parameter(name = "radius", description = "A search radius in miles", example = "10")
	})
	@Tag(name = "Search API")
	PagedCollection<List<Course>> searchByJurisdiction(
			@QueryParam("latitude") @DecimalMin("-90.0") @DecimalMax("90.0") double latitude,
			@QueryParam("longitude") @DecimalMin("-180.0") @DecimalMax("180.0") double longitude,
			@QueryParam("radius") @Positive @Max(100) Integer radiusInMiles, @BeanParam @Valid Pagination paging);

}
