package com.poststats.golf.rs.api;

import com.poststats.golf.rs.api.model.Event;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.security.RolesAllowed;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.POST;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;

/**
 * @author brian.long@poststats.com
 */
@Path("/golf/event/{eventId}")
@Tag(name = "Event API")
public interface EventApi {

	@GET
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Retrieves limited meta-data about an event.",
			description = "Retreives name, location, dates, and other direct meta-data about the specified event."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	Event get(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/detail")
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Retrieves detailed meta-data about an event.",
			description = "Retreives name, location, dates, courses, and other indirect meta-data about the specified event."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	Event getDetail(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@POST
	@Path("/document/{documentId}/send")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "communicate"
	)
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Sends the specified document.",
			description = "Sends the specified document off-schedule, regardless of when it is configured to send."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event or document with the specified ID could not be found"
			)
	})
	void sendDocument(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for an event document")
			@NotNull @Positive @PathParam("documentId") long documentId);

	@POST
	@Path("/document/{documentId}/sendTest/{personId}")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "communicate"
	)
	@Produces(Constants.V1_JSON)
	@Operation(
			summary = "Sends the specified document to the specified person.",
			description = "Sends the specified document to only the specified person off-schedule."
	)
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(
					responseCode = "404",
					description = "An event, document, or person with the specified ID could not be found"
			)
	})
	void sendTestDocument(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for an event document")
			@NotNull @Positive @PathParam("documentId") long documentId,
			@Parameter(description = "A unique identifier for a person")
			@NotNull @Positive @PathParam("personId") long personId);

}
