package com.poststats.golf.rs.api;

import java.util.List;
import java.util.Map;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.security.RolesAllowed;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.QueryParam;
import jakarta.ws.rs.core.StreamingOutput;

@Path("/golf/event/{eventId}/finance")
@Tag(name = "Event Finance API")
public interface EventFinanceApi {

	@GET
	@Path("/balance/persons")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "member"
	)
	@Produces(Constants.V1_JSON)
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves the balances of all persons associated with an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	List<Map<String, Object>> getBalanceByPersonsAsJson(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId, @QueryParam("minBalance") Float minBalance,
			@QueryParam("maxBalance") Float maxBalance);

	@GET
	@Path("/balance/persons/csv")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "finance"
	)
	@Produces("text/csv")
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves the balances of all persons associated with an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	StreamingOutput getBalanceByPersonsAsCsv(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/balance/person/{personId}")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "member"
	)
	@Produces(Constants.V1_JSON)
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves the balance of a person in an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(
					responseCode = "404",
					description = "An event or person with the specified IDs could not be found"
			)
	})
	Map<String, Object> getBalanceByPersonsAsJson(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId,
			@Parameter(description = "A unique identifier for a person")
			@NotNull @Positive @PathParam("personId") long personId);

	@GET
	@Path("/series/balance/persons")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "member"
	)
	@Produces(Constants.V1_JSON)
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves the cumulative balances of all persons associated with a series prior to an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	List<Map<String, Object>> getSeriesBalanceByPersonsAsJson(
			@Parameter(description = "A unique identifier for an event")
			@NotNull @Positive @PathParam("eventId") long eventId);

}
