package com.poststats.golf.rs.api;

import java.util.List;
import java.util.Set;

import com.poststats.golf.rs.api.model.EventPerson;

import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.responses.ApiResponse;
import io.swagger.v3.oas.annotations.responses.ApiResponses;
import io.swagger.v3.oas.annotations.security.SecurityRequirement;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.security.RolesAllowed;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Positive;
import jakarta.ws.rs.GET;
import jakarta.ws.rs.Path;
import jakarta.ws.rs.PathParam;
import jakarta.ws.rs.Produces;
import jakarta.ws.rs.core.StreamingOutput;

@Path("/golf/event/{eventId}")
@Tag(name = "Event Participant API")
public interface EventPersonApi {

	@GET
	@Path("/people")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "member"
	)
	@Produces(Constants.V1_JSON)
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves limited meta-data about all the participants and administrators in an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	List<? extends EventPerson> get(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/people/detail")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "member"
	)
	@Produces(Constants.V1_JSON)
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves detailed meta-data about all the participants and administrators in an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	List<? extends EventPerson> getDetail(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/people/csv")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "membership"
	)
	@Produces("text/csv")
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves address book meta-data about all the participants and administrators in an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	StreamingOutput getAsCsv(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/participants")
	@Produces(Constants.V1_JSON)
	@Operation(summary = "Retrieves limited meta-data about all the participants in an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	List<? extends EventPerson> getParticipants(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/participants/csv")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "membership"
	)
	@Produces("text/csv")
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves address book meta-data about all the participants in an event.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	StreamingOutput getParticipantsAsCsv(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

	@GET
	@Path("/series/participants")
	@RolesAllowed(
		Constants.EVENT_ROLE_PREFIX
				+ "membership"
	)
	@Produces(Constants.V1_JSON)
	@SecurityRequirement(name = "basic")
	@Operation(summary = "Retrieves limited meta-data about all the participants in an event series.")
	@ApiResponses({
			@ApiResponse(responseCode = "200", description = "Success"),
			@ApiResponse(responseCode = "401", description = "Not authenticated"),
			@ApiResponse(responseCode = "403", description = "Authenticated, but not permitted"),
			@ApiResponse(responseCode = "404", description = "An event with the specified ID could not be found")
	})
	Set<Long> getSeriesParticipants(@Parameter(description = "A unique identifier for an event")
	@NotNull @Positive @PathParam("eventId") long eventId);

}
